#!/usr/bin/env python
"""Python binding for ditto compressor
"""
import os
import subprocess


def ditto(event_sequences):
    running_dir = os.getcwd()

    # create a .dat file from the data in data dir of ditto source
    ditto_dir = os.path.join(running_dir, 'compressor', 'ditto')
    os.chdir(ditto_dir)

    data_path = 'data.dat'
    num_event_sequences = len(event_sequences)
    alphabet_sizes = [max(event_sequence) +
                      1 for event_sequence in event_sequences]
    alphabet_sizes_raw = " ".join(str(v) for v in alphabet_sizes)
    header = "%d %s" % (num_event_sequences, alphabet_sizes_raw)
    event_sequences_text = [" ".join(str(v) for v in event_sequence)
                            for event_sequence in event_sequences]
    body = " -2\n".join(event_sequence_text for event_sequence_text in event_sequences_text)
    content = header + "\n" + body

    with open(data_path, "w") as writer:
        writer.write(content)

    # run ditto
    ditto_executable = './ditto'
    ditto_cmd = [ditto_executable, '-i', data_path, '-m', '20']
    output = subprocess.check_output(ditto_cmd, stderr=subprocess.STDOUT)
    compressed_size = float(output)

    os.chdir(running_dir)

    return compressed_size, None

if __name__ == "__main__":
    event_sequences = [range(10), range(10)]
    ditto(event_sequences)
