import sys
import os
from math import log
from mdl import binomial_mdl
from out import out_dag, out_scheme, out_sets, out_model
import profile


bmdl = None


def binmdl(n):
    global bmdl
    if bmdl[n] == -1:
        bmdl[n] = binomial_mdl(n)
    return bmdl[n]


def isin(v, t):
    s = 0
    e = len(t) - 1

    while (s <= e):
        m = (s + e) / 2
        if t[m] == v:
            return True
        elif t[m] < v:
            s = m + 1
        elif t[m] > v:
            e = m - 1

    return False


def split(v, data):
    p = []
    n = []

    for t in data:
        if isin(v, t):
            p.append(t)
        else:
            n.append(t)
    return [p, n]


class Tree:

    def __init__(s):
        s.leaf = True
        s.neg = None
        s.pos = None
        s.attr = None
        s.score = 0
        s.saved = 0
        s.total = 0
        s.ncount = 0
        s.pcount = 0

    def split(s, neg, pos, attr):
        s.leaf = False
        s.neg = neg
        s.pos = pos
        s.attr = attr

    def update_total(s):
        if s.leaf == False:
            s.total = s.score + s.neg.update_total() + s.pos.update_total()
        else:
            s.total = s.score
        return s.total


class Scheme:

    def __init__(s, data, target, cands, K):
        s.target = target
        s.tree = Tree()
        s.fragments = [Fragment([[]], split(target, data), cands, K, s.tree)]
        s.update_scores()

    def update_scores(s):
        s.scores = [s.fragments[i].best_score() + [i]
                    for i in range(len(s.fragments))]
        s.best_score = min(s.scores)

    def remove_cands(s, black):
        for f in s.fragments:
            f.remove_cands(black)
        s.update_scores()

    def split(s):
        i = s.best_score[2]
        s.fragments[i].tree.saved = -s.best_score[0]
        s.fragments = s.fragments[
            0:i] + s.fragments[i].split(s.best_score[1]) + s.fragments[(i + 1):]
        s.update_scores()

    def out(s):
        print s.target, ':'
        for f in s.fragments:
            f.out()


class Fragment:

    def __init__(s, fr, data, cands, K, tree):
        s.data = data
        s.fr = fr
        s.K = K
        s.cands = cands[:]
        s.tree = tree

        s.cm = [-1] * s.K
        for i in range(len(s.cands)):
            s.cm[s.cands[i]] = i

        s.tree.score = s.bits()
        s.tree.pcount = len(data[0])
        s.tree.ncount = len(data[1])
        s.score = s.calculate_score(s.calculate_freqs())

    def calculate_freqs(s):
        freqs = [[0] * len(s.cands), [0] * len(s.cands)]

        for t in s.data[0]:
            for i in t:
                if s.cm[i] >= 0:
                    freqs[0][s.cm[i]] += 1

        for t in s.data[1]:
            for i in t:
                if s.cm[i] >= 0:
                    freqs[1][s.cm[i]] += 1
        return freqs

    def remove_cands(s, black):
        for k in black:
            if s.cm[k] >= 0:
                s.cands[s.cm[k]] = -1
                s.score[s.cm[k]] = 1
        return

    def bits(s):
        entr = lambda x: 0 if x == 0 else -x * log(x, 2)
        pen = lambda x: binmdl(int(x)) + 1
        f = [len(s.data[0]), len(s.data[1])]
        n = sum(f)
        # A dirty trick to care of zero values.
        f = [x + int(x == 0) for x in f]
        return sum(map(entr, f)) - entr(n) + pen(n)

    def split_bits(s, f):
        n1 = [f[0] + f[2], f[1] + f[3]]
        n2 = [f[0] + f[1], f[2] + f[3]]
        n = [n1[0] + n1[1]]

        if (min(n1) == 0 or min(n2) == 0):
            return 1

        # A dirty trick to care of zero values.
        f = [x + int(x == 0) for x in f]

        pen2 = 1 + log(s.K, 2)

        return entropy(f) - entropy(n1) - entropy(n2) + entropy(n) + penalty(n1) - penalty(n) + pen2

    def calculate_score(s, freqs):
        score = [0] * len(s.cands)

        for i in range(len(s.cands)):
            f = [0] * 4
            f[0] = float(freqs[0][i])
            f[1] = float(len(s.data[0]) - freqs[0][i])
            f[2] = float(freqs[1][i])
            f[3] = float(len(s.data[1]) - freqs[1][i])
            score[i] = s.split_bits(f)

        return score

    def best_score(s):
        if len(s.score) == 0:
            return [1, -1]
        i = s.score.index(min(s.score))
        return [s.score[i], s.cands[i]]

    def split(s, v):
        datap = split(v, s.data[0])
        datan = split(v, s.data[1])
        cands = filter(lambda x: x >= 0 and x <> v, s.cands)

        neg = Tree()
        pos = Tree()
        s.tree.split(neg, pos, v)
        s.tree.score = 1 + log(s.K, 2)
        frsets = [x + [v] for x in s.fr]
        fr1 = Fragment(frsets, [datap[0], datan[0]], cands, s.K, pos)
        fr2 = Fragment(s.fr + frsets, [datap[1], datan[1]], cands, s.K, neg)
        return [fr1, fr2]

    def out(s):
        print s.fr
        # sys.stdout.write('[');
        # for f in s.fr:
        # if f < 0:
        # sys.stdout.write('-');
        # else:
        # sys.stdout.write('+');
        # sys.stdout.write(str(abs(f)-1));
        #sys.stdout.write(', ');
        # print ']';


def entropy(x):
    res = 0
    for y in x:
        res -= y * log(y, 2)
    return res


def penalty(x):
    res = 0
    for y in x:
        res += binmdl(int(y)) + 1
    return res


class Dag:

    def __init__(s, K):
        s.edges = [set() for i in range(K)]
        s.children = [set() for i in range(K)]

    def add(s, par, child):
        s.edges[child] |= set([par])
        s.update(par, s.children[child] | set([child]))

    def update(s, node, children):
        if not children <= s.children[node]:
            s.children[node] |= children
            for c in s.edges[node]:
                s.update(c, children)


def build_trees(data, K, partition_of_dimensions=None):
    if partition_of_dimensions:
        schemes = []
        for subset_of_dimensions in partition_of_dimensions:
            for dimension in subset_of_dimensions:
                candidates = list(
                    set(subset_of_dimensions).difference([dimension]))
                print dimension, ":", candidates
                scheme = Scheme(data, dimension, candidates, K)
                schemes.append(scheme)
    else:
        schemes = [Scheme(data, i, range(i) + range(i + 1, K), K)
                   for i in range(K)]
    deps = Dag(K)

    orig_score = sum([s.tree.score for s in schemes])

    while True:
        scores = [schemes[i].best_score + [i] for i in range(K)]
        bs = min(scores)

        # print '%.3f at %d with %d' % (bs[0], bs[3], bs[1]);

        if bs[0] > 0:
            break

        schemes[bs[3]].split()

        # print 'updating graph';
        deps.add(bs[1], bs[3])
        # print 'removing cands';
        for i in range(len(schemes)):
            schemes[i].remove_cands(deps.children[i])

    return (schemes, orig_score, deps)


def pack(observations, partition_of_dimensions=None):
    # items must start with 0
    global bmdl
    try:
        K = 1 + max(map(max, filter(bool, observations)))
    except ValueError:
        return 1.0, []
    bmdl = [-1] * (1 + len(observations))
    schemes, orig_score, deps = build_trees(
        observations, K, partition_of_dimensions)
    for s in schemes:
        s.tree.update_total()
    total_score = sum([s.tree.total for s in schemes])
    for attr, edge in enumerate(deps.edges):
        if edge:
            edge.update([attr])
    model = deps.edges
    return total_score, model


if __name__ == '__main__':
    dataPath = '/Users/mbk-97-51/Projects/coco/code/src/data/synthetic_r100_a100_n0.05.dat'
    dataPath = '/Users/kbudhath/Projects/dc/data/datasets/dat-files/mushroom.dat'
    with open(dataPath, 'r') as reader:
        newline_data_str = reader.readlines()
        data = [map(int, line.split()) for line in newline_data_str]

    print pack(data)
