#!/usr/bin/env python
# -*- coding: utf-8 -*-
import glob
import math
import os
import re
import shutil
import subprocess


def log2(n):
    return math.log(n, 2)


def log2_binomial(n, k):
    if k == 0 or n == k:
        return 0

    assert n > 0 and k > 0
    return n * log2(n) - k * log2(k) - (n - k) * log2(n - k)


def slim(observations):
    num_empty_obs = 0
    for obs in observations:
        if not obs:
            num_empty_obs += 1
    correction = log2_binomial(len(observations), num_empty_obs)

    running_dir = os.getcwd()

    # create a .dat file from the data in data dir
    slim_dir = os.path.join(running_dir, 'compressor', 'slim')
    os.chdir(slim_dir)

    data_path = os.path.join('data', 'datasets', 'temp', 'temp.dat')
    with open(data_path, "w") as writer:
        for observation in observations:
            row = " ".join(str(item) for item in observation)
            writer.write(row + "\n")

    # convert .dat file to .db file using convertdb.conf
    # we assume that the data will always be in temp directory
    fic_executable = './fic'
    convert_conf = 'convertdb.conf'
    convert_cmd = [fic_executable, convert_conf]
    subprocess.check_output(convert_cmd, stderr=subprocess.STDOUT)

    # run slim with compress.conf and get the result from the standard output
    compress_conf = 'compress.conf'
    compress_cmd = [fic_executable, compress_conf]
    output = subprocess.check_output(compress_cmd, stderr=subprocess.STDOUT)

    # extract the compressed size from stdout
    match = re.search(
        r'\* Result:\s+\([a-zA-Z0-9]*,[a-zA-Z0-9]*,\d+,\d+,\d+,(\d+)\)', output)
    # When probability of cover of transaction is 1, SLIM will throw
    # this exception "DoeJeDing - L(D|M) < 0. That's not good.". We implicitly
    # assume that any error in SLIM is only this and assign 0 bits.
    compressed_size = float(match.group(1)) if match else 0
    compressed_size += correction #+ log2(len(observations))

    # find the code table file
    outdir = os.path.join('xps', 'compress_ng')
    all_subdirs = [os.path.join(outdir, d) for d in os.listdir(
        outdir) if os.path.isdir(os.path.join(outdir, d))]
    latest_subdir = max(all_subdirs, key=os.path.getmtime)
    ct_file_pat = latest_subdir + os.sep + 'ct-temp-all-1d-0-[1-9]*'
    matches = glob.glob(ct_file_pat)
    ct_file = matches[0] if matches else None

    # open the db file and convert the fic ids to fimi ids from ct
    db_file = os.path.join('data', 'datasets', 'temp', 'temp.db')
    fic_ids = []
    fimi_ids = []
    with open(db_file, 'r') as reader:
        for line in reader:
            if line.startswith('ab'):
                fic_ids = line[3:].strip().split()
            elif line.startswith('it'):
                fimi_ids = line[3:].strip().split()
    translator = dict(zip(map(int, fic_ids), map(int, fimi_ids)))

    edge_groups = []
    if ct_file:
        with open(ct_file, 'r') as reader:
            reader.readline()
            reader.readline()
            for line in reader:
                left, right = line.split('(')
                nodes = left.rstrip().split()
                nodes = [translator[int(node)] for node in nodes]
                if len(nodes) > 1:
                    group = set(nodes)
                    edge_groups.append(group)

    shutil.rmtree(latest_subdir)  # clean up the output
    os.chdir(running_dir)
    model = edge_groups
    return compressed_size, model
