#!/usr/bin/env python
# -*- coding: utf-8 -*-
import matplotlib as mp
import matplotlib.cm as cm
import matplotlib.pyplot as plt


def _configure_matplotlib():
    plt.style.use('ggplot')
    font_size = 13
    fig = mp.pyplot.gcf()
    fig.set_size_inches(18, 12)


_configure_matplotlib()


def plot_multiline(Ys, X, labels, xlabel, ylabel, title, save_fig=False, fig_name=None, linestyles=None, invert_x_axis=False):
    plt.title(title)
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)

    for i, Y in enumerate(Ys):
        linestyle = linestyles[i] if linestyles else "-"
        plt.plot(X, Y, label=labels[i], marker="o", linewidth=2.0,
                 dash_joinstyle="bevel", solid_capstyle="round", markeredgecolor="none", linestyle=linestyle)
        plt.legend(loc="upper left")
    plt.ylim(0.0, 1.001)
    if invert_x_axis:
        plt.xlim(max(X), min(X))
    if save_fig:
        plt.savefig("figs/%s" % fig_name)
    else:
        plt.show()


def plot_scatter(Y, X, xlabel, ylabel, title, save_fig=False, fig_name=None, colors=[]):
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)

    plt.scatter(X, Y, color='r')

    if save_fig:
        plt.savefig("figs/%s" % fig_name)
    else:
        plt.show()
