#!/usr/bin/env python
# -*- coding: utf-8 -*-
import itertools
import random
import sys
import time

from tools import *


CACHE = {}


def clear_cache():
    global CACHE
    CACHE = {}


def cache(store, dimension_names, compressor):
    global CACHE
    key = frozenset(dimension_names)
    if key in CACHE:
        return CACHE.get(key), None
    else:
        observations = store.project(dimension_names)
        size, _ = compressor(observations)
        CACHE[key] = size
        return size, None


def total_algorithmic_correlation(store, compressor, dimension_names=None, use_cache=False):
    dimension_names = dimension_names or store.dimension_names
    if use_cache:
        together, _ = cache(store, dimension_names, compressor)
    else:
        full_projection = store.project(dimension_names)
        together, _ = compressor(full_projection)

    min_individual = sys.float_info.max
    sum_individual = 0
    for dimension_name in dimension_names:
        if use_cache:
            projection_size, _ = cache(store, [dimension_name], compressor)
        else:
            projection = store.project([dimension_name])
            projection_size, _ = compressor(projection)
        # print projection_size,
        if projection_size < min_individual:
            min_individual = projection_size
        sum_individual += projection_size

    # print "\nmin_ind =", min_individual
    # print "sum_ind =", sum_individual
    # print "together =", together
    # print "sum_ind - min_ind =", (sum_individual - min_individual)
    # print "sum_ind - together =", (sum_individual - together)

    score = (sum_individual - together) * 1.0 / \
        (sum_individual - min_individual)
    return score


def total_singular_algorithmic_correlation(store, compressor, dimension_names=None, use_cache=False):
    dimension_names = dimension_names or store.dimension_names
    if use_cache:
        together, _ = cache(store, dimension_names, compressor)
    else:
        full_projection = store.project(dimension_names)
        together, _ = compressor(full_projection)
    # print
    sum_individual = 0
    for dimension_name in dimension_names:
        if use_cache:
            projection_size, _ = cache(store, [dimension_name], compressor)
        else:
            projection = store.project([dimension_name])
            projection_size, _ = compressor(projection)
        # print projection_size,
        sum_individual += projection_size

    # print
    sum_individual_conditional = 0
    for dimension_name in dimension_names:
        other_dimension_names = list(dimension_names)
        other_dimension_names.remove(dimension_name)
        if use_cache:
            projection_size, _ = cache(
                store, other_dimension_names, compressor)
        else:
            projection = store.project(other_dimension_names)
            projection_size, _ = compressor(projection)
        individual_conditional = together - projection_size
        # print individual_conditional,
        sum_individual_conditional += individual_conditional

    # print "\ntogether =", together
    # print "sum_ind =", sum_individual
    # print "sum_ind_cond =", sum_individual_conditional
    # print
    score = (sum_individual - sum_individual_conditional) * \
        1.0 / sum_individual
    return score
