#!/usr/bin/env python
# -*- coding: utf-8 -*-
from operator import itemgetter
import os
import sys
import time

from pybloom import BloomFilter

from compressor import pack
from score import clear_cache, total_algorithmic_correlation, total_singular_algorithmic_correlation
from storage import DiscreteStore


def spinning_cursor():
    while True:
        for cursor in '|/-\\':
            yield cursor


def read_file(data_path):
    with open(data_path, 'r') as reader:
        rows = reader.readlines()
    rows = [row.split() for row in rows]
    rows = [map(int, row) for row in rows if row]
    return rows


def generate_candidates_slim(patterns, items, generated):
    candidates = []
    for i in xrange(len(patterns)):
        for j in xrange(i + 1, len(patterns)):
            candidate = patterns[i].union(patterns[j])
            if candidate in generated:
                continue
            candidates.append(candidate)
            generated.add(candidate)
    return candidates


def generate_candidates_apriori(patterns, items, generated):
    candidates = []
    for pattern in patterns:
        for item in items:
            if item in pattern:
                continue

            candidate = pattern.union({item})
            candidate = sorted(list(candidate))
            cand_key = "".join(str(id) for id in candidate)
            if cand_key in generated:
                continue

            candidates.append(candidate)
            generated.add(cand_key)
    return candidates


def levelwise_search(store, compressor, threshold, score_func, cand_gen_func=generate_candidates_apriori, release=False, min_sup=0, max_level=-1):
    start = time.time()
    spinner = spinning_cursor()
    generated = BloomFilter(capacity=2**32, error_rate=0.001)
    freq_dimensions = [
        d for d in store.dimension_names if store.dimension_supp[d] >= min_sup]
    patterns = [{d} for d in freq_dimensions]

    level = 1
    num_tested = 0
    num_discovered = 0
    discovered = []
    max_score = 0.0
    while patterns:
        if max_level != -1 and max_level == level:
            break
        new_patterns = []
        candidates = cand_gen_func(patterns, freq_dimensions, generated)
        for candidate in candidates:
            if release:
                sys.stdout.write(spinner.next() + " %d (%d)" %
                                 (num_discovered, num_tested))
                sys.stdout.flush()

            score = score_func(store, compressor, candidate, True)
            if score > max_score:
                max_score = score
            if score >= threshold:
                num_discovered += 1
                new_patterns.append(set(candidate))
                discovered.append((candidate, score))
                if not release:
                    print "{%s} %.2f" % (" ".join(str(v) for v in candidate), score)
            num_tested += 1
            if release:
                sys.stdout.write('\b' * 100)
        level += 1
        print "\nlevel=%d num_patterns=%d" % (level, len(discovered))
        patterns = new_patterns
    end = time.time()
    print "max_score =", max_score
    print "time = %.2f" % (end - start)
    discovered = sorted(discovered, key=itemgetter(1))
    clear_cache()
    return discovered


if __name__ == '__main__':
    data = 'icdm'
    data_path = '/Users/kbudhath/Projects/wiggle/data/%s.dat' % data
    rows = read_file(data_path)
    store = DiscreteStore(rows)
    score_func = total_algorithmic_correlation
    discovered = levelwise_search(store, pack, 0.5, score_func, release=False)
    print "num_patterns = %d" % len(discovered)
    labels_path = '/Users/kbudhath/Projects/wiggle/data/%s.labels' % data
    with open(labels_path, "r") as reader:
        labels = dict(enumerate(map(str.rstrip, reader.readlines())))
    for pat in discovered:
        pat_str = " ".join(labels[item] for item in pat[0])
        print "%s → %.2f" % (pat_str, pat[1])
    # with open("output/%s-opt-ind.dat" % data, "w") as writer:
    #     for d in discovered:
    #         if len(d) > 2:
    #             writer.write("%.2f %.2f\n" % (d[1], d[2]))
