#!/usr/bin/env python
# -*- coding: utf-8 -*-
from compressor import pack, slim
from search import levelwise_search, read_file, generate_candidates_slim, generate_candidates_apriori
from score import total_algorithmic_correlation, total_singular_algorithmic_correlation
from storage import DiscreteStore


def test_qualitative_datasets():
    threshold = 0.5
    compressor = pack
    datasets = ['mammals']
    score_func = total_algorithmic_correlation
    for dataset in datasets:
        print "%s" % dataset.upper()
        data_path = 'data/%s.dat' % dataset
        rows = read_file(data_path)
        store = DiscreteStore(rows)

        discovered = levelwise_search(
            store, compressor, threshold, score_func, cand_gen_func=generate_candidates_apriori, release=True, min_sup=0)
        print "num_patterns = %d" % len(discovered)
        labels_path = 'data/%s.labels' % dataset
        with open(labels_path, "r") as reader:
            labels = dict(enumerate(map(str.rstrip, reader.readlines())))
        for pat in discovered:
            pat_str = " ".join(labels[item] for item in pat[0])
            print "%s %s → %.2f" % (str(pat), pat_str, pat[1])
        print


def test_quantitative_datasets():
    threshold = 0.85
    compressor = pack
    score_func = total_algorithmic_correlation
    datasets = ['adult', 'anneal', 'breast',
                'chess', 'led7', 'mushroom']
    for dataset in datasets:
        data_path = "data/%s.dat" % dataset
        observations = read_file(data_path)
        store = DiscreteStore(observations)
        discovered = levelwise_search(
            store, compressor, threshold, score_func, cand_gen_func=generate_candidates_apriori, release=True, min_sup=0, max_level=4)
        print "%s → %d" % (dataset, len(discovered))

# test_qualitative_datasets()
test_quantitative_datasets()
